package paldirvd;

/***/

public class Position {
    
    private double x, y, z, error;
    private double dec, inc, intensity;
    
    Position (double a, double b, double c, double err)
    {
        x = a; // vertical (x<0 lower hemisphere -> inclination positive)
        y = b; // north
        z = c; // east
        error = err;
        intensity();
        decInc();
    }
    Position (double declination, double inclination, double err)
    {
        dec = declination;
        inc = inclination;
        error = err;
        calculate_xyz();
        intensity(); //will be 1
    }
    Position (double a, double b, double c, double intens, double err)
    {
        x = a; // vertical
        y = b; // north
        z = c; // est
        error = err;
        intensity = intens;
        decInc();
    }

    public void setInt(double inten){
        intensity =inten; x=x*intensity; y=y*intensity; z=z*intensity;
    }
        
    private void intensity ()
    {
        intensity = Math.sqrt(x*x + y*y + z*z);
    }
    
    private void decInc ()
    {
        if(y == 0)
            if(z > 0) 
                dec = 90;
            else      
                dec = 270;
        else{ 
            dec = Math.atan( z / -y ) * 180/Math.PI; 
            if(y > 0)
                dec = dec + 180;
            else if(dec < 0)
                dec = dec + 360;
        }
        if(y == 0 && z == 0)
            if(x > 0) 
                inc = 90;
            else      
                inc = -90;
        else
            inc = - Math.atan( x / Math.sqrt(y*y + z*z) ) * 180/Math.PI;
    }
    
     
    private void calculate_xyz (){
        double d = Math.toRadians(dec);
        double i = Math.toRadians(inc);
        x = - Math.sin(i);
        y = - Math.cos(i)*Math.cos(d);
        z = Math.cos(i)*Math.cos(d - Math.PI/2);
    } 
   
    double getX () {return x;}
    double getY () {return y;}
    double getZ () {return z;}
    double getError () {return error;}
    double getDec () {return dec;}
    double getInc () {return inc;}
    double getInt () {return intensity;}

    
    public void round4(){
        x = Math.round(x*10000)/10000d;
        y = Math.round(y*10000)/10000d;
        z = Math.round(z*10000)/10000d;
        
        dec = Math.round(dec*10000)/10000d;
        inc = Math.round(inc*10000)/10000d;
        error = Math.round(error*10000)/10000d;
        intensity = Math.round(intensity*10000)/10000d;
    }
    public void round(){
        dec = Math.round(dec);
        inc = Math.round(inc);
        error = Math.round(error*10)/10d;
        intensity = Math.round(intensity*10)/10d;
    }
    
    public Position normalize(){
        double a, b, c;
        intensity();
        a = x/intensity;
        b = y/intensity;
        c = z/intensity;
        Position pos = new Position(a,b,c,this.getError());
        return pos;
    }
   
    public Position inverse(){
        Position pos = new Position((dec+180)%360 , -inc, error);
        return pos;
    }
    
    public Position rot90(){
        double newInc = 90-inc;
        if(inc<0)
            newInc = -90-inc;
        Position pos = new Position((dec+90)%360 , newInc, error);
        return pos;
    }
}
