
package paldirvd;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Arc2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;
import javax.print.Doc;
import javax.print.DocFlavor;
import javax.print.DocPrintJob;
import javax.print.PrintException;
import javax.print.SimpleDoc;
import javax.print.StreamPrintService;
import javax.print.StreamPrintServiceFactory;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;


public class Print2DtoStream implements Printable{

    private Core core;
    private boolean tc, upNorth;
    private Vector inclCirc, exclCirc, inclDir, exclDir;
    
    private static int WIDTH = 250;
    private static int HEIGHT = 250; 
    
	public Print2DtoStream(String fileName, Core core_i, boolean tc_i, boolean upNorth_i) {
                
                core = core_i;
                tc = tc_i;
                if(tc){
                    core = core.beddingCorrection();
                }
                core = core.getValidCore();
                upNorth = upNorth_i;
                inclCirc = core.getInclCirc();
                exclCirc = core.getExclCirc();
                inclDir = core.getInclDir();
                exclDir = core.getExclDir();
                
		/* Use the pre-defined flavor for a Printable from an InputStream */
		DocFlavor flavor = DocFlavor.SERVICE_FORMATTED.PRINTABLE;

 		/* Specify the type of the output stream */
		String psMimeType = DocFlavor.BYTE_ARRAY.POSTSCRIPT.getMimeType();

		/* Locate factory which can export a GIF image stream as Postscript */
		StreamPrintServiceFactory[] factories =
		StreamPrintServiceFactory.lookupStreamPrintServiceFactories(
					flavor, psMimeType);
		if (factories.length == 0) {
			System.err.println("No suitable factories");
			System.exit(0);
		}

		try {
			/* Create a file for the exported postscript */
			FileOutputStream fos = new FileOutputStream(fileName);

			/* Create a Stream printer for Postscript */
			StreamPrintService sps = factories[0].getPrintService(fos);

			/* Create and call a Print Job */
			DocPrintJob pj = sps.createPrintJob();
			PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();

			Doc doc = new SimpleDoc(this, flavor, null);
			pj.print(doc, aset);
			fos.close();

		} catch (PrintException pe) { 
			System.err.println(pe);
		} catch (IOException ie) { 
			System.err.println(ie);
		}
	}
	
	public int print(Graphics g,PageFormat pf,int pageIndex) {

		if (pageIndex == 0) {
                  
                  data(g,pf);
                  g.translate((int)pf.getImageableX(), (int)pf.getImageableY());
                  zijderveldPlot(g);
                  g.translate(0, HEIGHT);
                  stereonet(g);
                  g.translate(0, HEIGHT+5);
                  intensityPlot(g);
                  boolean stacking = false;
                  if (core.getName().equals("STACKING")){
                    g.translate(WIDTH, 0);
                    errorPlot(g);}
                        return Printable.PAGE_EXISTS;			
		} else {
			return Printable.NO_SUCH_PAGE;
                }
	}
       
        private void data(Graphics g, PageFormat pf){
            boolean stacking = false;
            if (core.getName().equals("STACKING"))
                    stacking = true;
            int x0 = (int)pf.getImageableWidth() - 95;
            int y0 = (int)pf.getImageableY() + 10;
            int d = 15;
            g.drawString(core.getName(), x0, y0);
            y0 = y0 + d;
            
            int temp; Position pos; String line; VD vd;
            Vector v = core.getSteps();
            if(stacking){
                line = "Step, Dec, Inc, Int, Alfa, N";
                g.drawString(line, x0, y0 + d); y0 = y0 + d;
                line = "(ºC/mT) (º) (º) (E-6A/m)";
            }else{
                line = "Step, Dec, Inc, Int, Error";
                 g.drawString(line, x0, y0 + d); y0 = y0 + d;
                line = "(ºC/mT) (º) (º) (E-6A/m)";
            }
            g.drawString(line, x0, y0 + d);
            y0 = y0 + 2*d;
            for(int i=0; i<v.size(); i++){
                  temp = (int)((Step) v.get(i)).getTemp();
                  pos = ((Step) v.get(i)).getPos();
                  pos.round();
                  line = temp + ", " + (int)pos.getDec() + ", " + (int)pos.getInc() + ", " + pos.getInt() + ", " + pos.getError();
                  if(stacking)
                      line = line + ", " + ((Step) v.get(i)).getNumSteps();
                  g.drawString(line, x0, y0);
                  y0 = y0 + d;
            }
            if(inclDir.size()>0){
                line = "Inclor Directions";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                line = "Ini, End, Dec, Inc, Mad";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                for(int i=0; i<inclDir.size(); i++){
                    vd = (VD)inclDir.get(i);
                      pos = vd.getPos();
                      pos.round();
                      line = (int)vd.getStepIni() + ", " + (int)vd.getStepEnd() + ", " + (int)pos.getDec() + ", " + (int)pos.getInc() + ", " + pos.getError();
                      g.drawString(line, x0, y0);
                      y0 = y0 + d;
                }
            }
            if(exclDir.size()>0){
                line = "Exclor Directions";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                line = "Ini, End, Dec, Inc, Mad";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                for(int i=0; i<exclDir.size(); i++){
                    vd = (VD)exclDir.get(i);
                      pos = vd.getPos();
                      pos.round();
                      line = (int)vd.getStepIni() + ", " + (int)vd.getStepEnd() + ", " + (int)pos.getDec() + ", " + (int)pos.getInc() + ", " + pos.getError();
                      g.drawString(line, x0, y0);
                      y0 = y0 + d;
                }
            }
            if(inclCirc.size()>0){
                line = "Inclor Circles";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                line = "Ini, End, Dec, Inc, Mad";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                for(int i=0; i<inclCirc.size(); i++){
                    vd = (VD)inclCirc.get(i);
                      pos = vd.getPos();
                      pos.round();
                      line = (int)vd.getStepIni() + ", " + (int)vd.getStepEnd() + ", " + (int)pos.getDec() + ", " + (int)pos.getInc() + ", " + pos.getError();
                      g.drawString(line, x0, y0);
                      y0 = y0 + d;
                }                
            }
            if(exclCirc.size()>0){
                line = "Exclor Circles";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                line = "Ini, End, Dec, Inc, Mad";
                g.drawString(line, x0, y0 + d);
                y0 = y0 + 2*d;
                for(int i=0; i<exclCirc.size(); i++){
                    vd = (VD)exclCirc.get(i);
                      pos = vd.getPos();
                      pos.round();
                      line = (int)vd.getStepIni() + ", " + (int)vd.getStepEnd() + ", " + (int)pos.getDec() + ", " + (int)pos.getInc() + ", " + pos.getError();
                      g.drawString(line, x0, y0);
                      y0 = y0 + d;
                }                
            }
        }
        
        private void stereonet(Graphics g) {
              
              Dimension tam = new Dimension(WIDTH,HEIGHT); // plotPanel size
              int d = Math.min( tam.width,tam.height ) - 2; // diameter
              int x0 = (tam.width - d) / 2 + 1;
              int y0 = (tam.height - d) / 2 + 1;
              
              if (tc) {
                 g.drawString("TC", x0 + 15, y0 + 15);
              }else{
                 g.drawString("NO TC", x0 + 15, y0 + 15);
              }
    
              g.drawOval(x0, y0, d, d);// border circunference
              g.drawLine(x0+d/2, y0, x0+d/2, y0+d); 
              g.drawLine(x0, y0+d/2, x0+d, y0+d/2);
              
              g.setFont(new Font((g.getFont()).getFontName(), Font.PLAIN, 10));
              g.drawString("S", x0+d/2, y0+d); g.drawString("N", x0+d/2, y0+10);
              g.drawString("E", x0+d, y0+d/2); g.drawString("W", x0-5, y0+d/2);
              g.setFont(new Font((g.getFont()).getFontName(), Font.PLAIN, 12));
      
              x0 = x0 + d/2; y0 = y0 + d/2; // circunference centre     

              Step step; double dec, inc, r; int x=0, y=0, x_prev, y_prev;
   
              Vector v = core.getSteps();
              for(int i=0; i<v.size(); i++){
                  step = (Step) v.get(i);
                  dec = step.getPos().getDec();
                  inc = step.getPos().getInc();
                  dec = Math.toRadians(dec);
                  inc = Math.toRadians(inc);
                  r = d/2 * Math.sqrt(1 - Math.abs(Math.sin(inc)));
                  x_prev = x;
                  y_prev = y;
                  x = (int) Math.round(r * Math.sin(dec));
                  y = (int) Math.round(r * Math.cos(dec));
                  if(inc > 0)
                      g.fillOval(x+x0-d/90, -y+y0-d/90, d/45, d/45); // lower hemisphere
                  else
                      g.drawOval(x+x0-d/90, -y+y0-d/90, d/45, d/45); // upper hemisphere
                  if(i>=1)
                     g.drawLine(x_prev+x0, -y_prev+y0, x+x0, -y+y0);
              }

              //PLOT PCA 
              VD vd;
              g.setColor(Color.GRAY);
              for(int i=0; i<inclCirc.size(); i++){ //INCLOR
                vd = (VD)inclCirc.get(i);
                    Position pos = vd.getPos();
                    pos = pos.rot90();
                    plotArc((Graphics2D)g, pos.getDec(), pos.getInc(), d, x0, y0 );
              }
              for(int i=0; i<exclCirc.size(); i++){ //EXCLOR
                vd = (VD)exclCirc.get(i);
                    Position pos = vd.getPos();
                    pos = pos.rot90();
                    plotArc((Graphics2D)g, pos.getDec(), pos.getInc(), d, x0, y0 );
              }
               
              //Plot Drilling vector
              if(tc){
                dec = Math.toRadians(core.getTrendCorrected());
                inc = Math.toRadians(core.getPlungeCorrected());
              }else{          
                  dec = Math.toRadians(core.getTrend());
                  inc = Math.toRadians(core.getPlunge());
              }
              r = d/2 * Math.sqrt(1 - Math.abs(Math.sin(inc)));
              x = (int) Math.round(r * Math.sin(dec));
              y = (int) Math.round(r * Math.cos(dec));
              if(inc > 0)
                  g.fillOval(x+x0-d/90, -y+y0-d/90, d/45, d/45); // lower hemisphere
              else
                  g.drawOval(x+x0-d/90, -y+y0-d/90, d/45, d/45); // upper hemisphere
              
              g.setColor(Color.BLACK);
      
        }
        
       private void zijderveldPlot(Graphics g){ // ortogonal proyection resized

          Dimension tam = new Dimension (WIDTH,HEIGHT); // plotPanel size
          int dx = tam.width - 30; 
          int dy = tam.height - 30 ;
          int d = Math.min( tam.width,tam.height ) - 10; // diameter
          int x0 = 15;
          int y0 = 15;

          if (tc) {
             g.drawString("TC, ", x0, y0 + 10);
          }else{
             g.drawString("NO TC, ", x0, y0 + 10);
          }
          if(upNorth){
             g.drawString("up/N", x0 + 55, y0 + 10);    
          }else{
             g.drawString("up/W", x0 + 55, y0 + 10);
          }
          
          Vector v = core.getSteps(); 
          Step step; int a=0, b=0, c=0, a_prev, b_prev, c_prev;
          double a_max = 0, b_max = 0, c_max = 0, a_min = 0 , b_min = 0, c_min = 0, aux; // To resize
          double x_len, y_len; int x_min, y_min, x_max, y_max;
          for(int i=0; i<v.size(); i++){
             step = (Step) v.get(i);
             aux = (step.getPos().getX()); 
             if(aux > a_max)
                 a_max = aux;
             if(aux < a_min)
                 a_min = aux;
             aux = (step.getPos().getY()); 
             if(aux > b_max)
                 b_max = aux;
             if(aux < b_min)
                 b_min = aux;
             aux = (step.getPos().getZ()); 
             if(aux > c_max)
                 c_max = aux;
             if(aux < c_min)
                 c_min = aux;
          }
          a_min = Math.abs(a_min); b_min = Math.abs(b_min) ; c_min = Math.abs(c_min);
          // square centre resized to optimize the space
          if(upNorth){ // up/N
            x_len = c_max + c_min;   
            x_min = (int) (c_min * dx / x_len);
            x_max = (int) (c_max * dx / x_len);
            y_len = Math.max(b_min, a_max) + Math.max(b_max, a_min);
            y_max = (int) (Math.max(b_min, a_max) * dy / y_len);
            y_min = (int) (Math.max(b_max, a_min) * dy / y_len);
          }
          else{ // up/W
            x_len = b_max + b_min;
            x_min = (int) (b_max * dx / x_len);
            x_max = (int) (b_min * dx / x_len);
            y_len = Math.max(c_min, a_max) + Math.max(c_max, a_min);
            y_max = (int) (Math.max(c_min, a_max) * dy / y_len);
            y_min = (int) (Math.max(c_max, a_min) * dy / y_len);

          } 
          x0 = x0 + x_min; y0 = y0 + y_max; // square centre

          g.drawLine(x0, y0-y_max, x0, y0+y_min); // square axes
          g.drawLine(x0-x_min, y0, x0+x_max, y0);
        
          //labels
      g.setFont(new Font((g.getFont()).getFontName(), Font.PLAIN, 10));
      if(x_max>Math.abs(x_min)){
         g.drawLine(x0+(int)(x_max*0.9), y0-3, x0+(int)(x_max*0.9), y0+3);   
         g.drawString((int)(x_max*x_len/dx)+"E-6A/m", x0+x_max-55, y0-4);
      }else{
         g.drawLine(x0-(int)(x_min*0.9), y0-3, x0-(int)(x_min*0.9), y0+3);   
         g.drawString("-"+(int)(x_min*x_len/dx)+"E-6A/m", x0-x_min, y0-4);
      }
      if(y_max>Math.abs(y_min)){
         g.drawLine(x0-3, y0-(int)(y_max*0.9), x0+3, y0-(int)(y_max*0.9));
         g.drawString((int)(y_max*y_len/dy)+"E-6A/m", x0+5, y0-(int)(y_max*0.9)+4);
      }else{
         g.drawLine(x0-3, y0+(int)(y_min*0.9), x0+3, y0+(int)(y_min*0.9));
         g.drawString("-"+(int)(y_min*y_len/dy)+"E-6A/m", x0+5, y0+(int)(y_min*0.9)+4);
      }
      g.setFont(new Font((g.getFont()).getFontName(), Font.PLAIN, 12));
  
          for(int i=0; i<v.size(); i++){
             step = (Step) v.get(i);
             a_prev = a; b_prev = b; c_prev = c;

             if(upNorth){
                 a = (int) (step.getPos().getX() * 0.9 * dy / y_len);
                 b = (int) (step.getPos().getY() * 0.9 * dy / y_len);
                 c = (int) (step.getPos().getZ() * 0.9 * dx / x_len);
             }else{
                 a = (int) (step.getPos().getX() * 0.9 * dy / y_len);
                 b = (int) (step.getPos().getY() * 0.9 * dx / x_len);
                 c = (int) (step.getPos().getZ() * 0.9 * dy / y_len);
             }

             if( upNorth ){ // up/N
                g.fillOval(x0+c-d/90, y0-(-b)-d/90, d/45, d/45); // declination
                g.drawOval(x0+c-d/90, y0-a-d/90, d/45, d/45); // inclination
                if(i>=1){
                    g.drawLine(x0+c_prev, y0-a_prev, x0+c, y0-a);
                    g.drawLine(x0+c_prev, y0-(-b_prev), x0+c, y0-(-b));
                }
             }else{ // up/W
                g.fillOval(x0+(-b)-d/90, y0-(-c)-d/90, d/45, d/45); // declination
                g.drawOval(x0+(-b)-d/90, y0-a-d/90, d/45, d/45); // inclination
                if(i>=1){
                    g.drawLine(x0+(-b_prev), y0-(-c_prev), x0+(-b), y0-(-c));
                    g.drawLine(x0+(-b_prev), y0-a_prev, x0+(-b), y0-a);
                }
             }
           }

          //PLOT PCA 
          VD vd;
          g.setColor(Color.GRAY);
          double cte = 30;
          for(int i=0; i<inclDir.size(); i++){
          if(inclDir.size()>0){ //INCLOR
                vd = (VD)inclDir.get(i);
                    Position pos = vd.getPos();
                    if(upNorth){
                         a = (int) (pos.getX() * cte * dy / y_len);
                         b = (int) (pos.getY() * cte * dy / y_len);
                         c = (int) (pos.getZ() * cte * dx / x_len);

                         g.drawLine(x0-c, y0+a, x0+c, y0-a);
                         g.drawLine(x0-c, y0-b, x0+c, y0-(-b));
                     }else{
                         a = (int) (pos.getX() * cte * dy / y_len);
                         b = (int) (pos.getY() * cte * dx / x_len);
                         c = (int) (pos.getZ() * cte * dy / y_len);

                         g.drawLine(x0+b, y0+a, x0+(-b), y0-a);
                         g.drawLine(x0+b, y0-c, x0+(-b), y0-(-c));
                     }
           }
           }
           for(int i=0; i<exclDir.size(); i++){
           if(exclDir.size()>0){ //EXCLOR
                vd = (VD)exclDir.get(i);
                    Position pos = vd.getPos();
                    Position pos_mean = vd.getPosMean();
                    if(upNorth){
                         int amean = ((int)(pos_mean.getX() * 0.9 * dy /  y_len));
                         int bmean = ((int)(pos_mean.getY() * 0.9 * dy /  y_len));
                         int cmean = ((int)(pos_mean.getZ() * 0.9 * dx /  x_len));
                         a = (int) (pos.getX() * cte * dy /  y_len );
                         b = (int) (pos.getY() * cte * dy /  y_len );
                         c = (int) (pos.getZ() * cte * dx /  x_len );
                         b = -b; bmean = -bmean;
                         g.drawLine(x0+(cmean - c), y0-(amean - a), x0+(cmean + c), y0-(amean + a));
                         g.drawLine(x0+(cmean - c), y0-(bmean - b), x0+(cmean + c), y0-(bmean + b));
                     }else{
                         int amean = ((int)(pos_mean.getX() * 0.9 * dy /  y_len));
                         int bmean = ((int)(pos_mean.getY() * 0.9 * dx /  x_len));
                         int cmean = ((int)(pos_mean.getZ() * 0.9 * dy /  y_len));
                         a = (int) (pos.getX() * cte * dy /  y_len);
                         b = (int) (pos.getY() * cte * dx /  x_len);
                         c = (int) (pos.getZ() * cte * dy /  y_len);
                         b = -b; bmean = -bmean; c = -c; cmean = -cmean;
                         g.drawLine(x0+(bmean - b), y0-(amean - a), x0+(bmean + b), y0-(amean + a));
                         g.drawLine(x0+(bmean - b), y0-(cmean - c), x0+(bmean + b), y0-(cmean + c));
                     }
           }
           }
           g.setColor(Color.BLACK);
        }
        
        private void intensityPlot(Graphics g) {
         
          Dimension tam = new Dimension(WIDTH/2,HEIGHT/2); // panel size
          int d = Math.max(tam.height, tam.width) / 10;
          int x0 = d/2; int y0 = d/2; // nw corner

          g.drawString("Intensity (E-6 A/m)", x0 + 50, y0 + 5);

          
          double intMin = 10, intMax = 0, aux; int inten, temp, inten_prev, temp_prev;

          for(int i=0; i<core.getSteps().size(); i++){
              aux = core.getStepN(i).getPos().getInt();
              if(intMin > aux)
                  intMin = aux;
              if(intMax < aux)
                  intMax = aux;
          }
          double s_inten = (tam.height-d)/(intMax-intMin); // scale y-intensity
          double s_temp = (tam.width-d)/(core.getStepN(core.getSteps().size()-1).getTemp() + core.getStepN(0).getTemp()); // scale x-temperature

          y0 = y0 + (int)(intMax*s_inten); // centre

          g.drawLine(x0, y0, x0+tam.width-d, y0); // axes
          g.drawLine(x0, y0 + (int)(intMin*s_inten), x0, y0 - (int)(intMax*s_inten));

          inten = (int) (core.getStepN(0).getPos().getInt() * s_inten);
          temp = (int) (core.getStepN(0).getTemp() * s_temp);
          for(int i=0; i<core.getSteps().size(); i++){
              inten_prev = inten;
              temp_prev = temp;
              inten = (int) (core.getStepN(i).getPos().getInt() * s_inten);
              temp = (int) (core.getStepN(i).getTemp() * s_temp);

              g.fillOval(x0 + temp - d/10, y0 - inten - d/10, (int)(d/5), (int)(d/5)); // intensity vs temperature step

              if(i>0){
                  g.drawLine(x0 + temp_prev, y0 - inten_prev, x0 + temp, y0 - inten);
              }

          }
          
            
      //Draw scale
      g.setFont(new Font((g.getFont()).getFontName(), Font.PLAIN, 10));
      aux = (intMin+intMax)/2;
      g.drawLine(x0-5, y0-(int)(aux*s_inten), x0+5, y0-(int)(aux*s_inten));
      g.drawString(Integer.toString((int)aux), x0-5, y0-(int)(aux*s_inten)+11);
      
      g.drawLine(x0-5, y0-(int)(intMax*s_inten), x0+5, y0-(int)(intMax*s_inten));
      g.drawString(Integer.toString((int)intMax), x0-5, y0-(int)(intMax*s_inten)+11);
      
      g.drawLine(x0-5, y0-(int)(intMin*s_inten), x0+5, y0-(int)(intMin*s_inten));
      g.drawString(Integer.toString((int)intMin), x0-5, y0-(int)(intMin*s_inten)+11);
      
      double stepMin=core.getStepN(0).getTemp(); 
      double stepMax=core.getStepN(core.getSteps().size()-1).getTemp();
      double stepInt=core.getStepN((int)core.getSteps().size()/2).getTemp();

      g.drawLine(x0+(int)(stepInt*s_temp),y0-5,x0+(int)(stepInt*s_temp),y0+5);
      g.drawString(Integer.toString((int)stepInt), x0+(int)(stepInt*s_temp), y0-1);
      
      g.drawLine(x0+(int)(stepMax*s_temp),y0-5, x0+(int)(stepMax*s_temp),y0+5);
      g.drawString(Integer.toString((int)stepMax), x0+(int)(stepMax*s_temp), y0-1);
      
      g.drawLine(x0+(int)(stepMin*s_temp), y0-5, x0+(int)(stepMin*s_temp), y0+5);
      g.drawString(Integer.toString((int)stepMin), x0+(int)(stepMin*s_temp), y0-1);
      g.setFont(new Font((g.getFont()).getFontName(), Font.PLAIN, 12));
      }
        
        
      private void errorPlot(Graphics g) {
     
          Dimension tam = new Dimension(WIDTH/2,HEIGHT/2); // panel size
          int d = Math.max(tam.height, tam.width) / 10;
          int x0 = d/2; int y0 = d/2; // nw corner

          g.drawString("Error", x0 + 50, y0 + 5);

      double errMin = 10, errMax = 0, aux; int error, temp, error_prev, temp_prev;
       
      core = core.getValidCore();
      for(int i=0; i<core.getSteps().size(); i++){
          aux = core.getStepN(i).getPos().getError();
          if(errMin > aux)
              errMin = aux;
          if(errMax < aux)
              errMax = aux;
      }
      double s_error = (tam.height-d)/(errMax); // scale y-error
      double s_temp = (tam.width-d)/(core.getStepN(core.getSteps().size()-1).getTemp() + core.getStepN(0).getTemp()); // scale x-temperature
              
      y0 = y0 + (int)(errMax*s_error); // centre
      
      g.drawLine(x0, y0, x0+tam.width-d, y0); // axes
      g.drawLine(x0, y0 , x0, y0 - (int)(errMax*s_error));
      
      error = (int) (core.getStepN(0).getPos().getError() * s_error);
      temp = (int) (core.getStepN(0).getTemp() * s_temp);
      for(int i=0; i<core.getSteps().size(); i++){
          error_prev = error;
          temp_prev = temp;
          error = (int) (core.getStepN(i).getPos().getError() * s_error);
          temp = (int) (core.getStepN(i).getTemp() * s_temp);
          
          if(i>0){
              g.drawLine(x0 + temp_prev, y0 - error_prev, x0 + temp, y0 - error);
          }
               
      }
      
      //Draw scale
      g.setFont(new Font((g.getFont()).getFontName(), Font.PLAIN, 10));
      aux = (errMin+errMax)/2;
      g.drawLine(x0-5, y0-(int)(aux*s_error), x0+5, y0-(int)(aux*s_error));
      g.drawString(Integer.toString((int)aux), x0-11, y0-(int)(aux*s_error)+11);
      
      g.drawLine(x0-5, y0-(int)(errMax*s_error), x0+5, y0-(int)(errMax*s_error));
      g.drawString(Integer.toString((int)errMax), x0-11, y0-(int)(errMax*s_error)+11);
      
      g.drawLine(x0-5, y0-(int)(errMin*s_error), x0+5, y0-(int)(errMin*s_error));
      g.drawString(Integer.toString((int)errMin), x0-11, y0-(int)(errMin*s_error)+11);

      double stepMin=core.getStepN(0).getTemp(); 
      double stepMax=core.getStepN(core.getSteps().size()-1).getTemp();
      double stepInt=core.getStepN((int)core.getSteps().size()/2).getTemp();

      g.drawLine(x0+(int)(stepInt*s_temp),y0-5,x0+(int)(stepInt*s_temp),y0+5);
      g.drawString(Integer.toString((int)stepInt), x0+(int)(stepInt*s_temp), y0-1);
      
      g.drawLine(x0+(int)(stepMax*s_temp),y0-5, x0+(int)(stepMax*s_temp),y0+5);
      g.drawString(Integer.toString((int)stepMax), x0+(int)(stepMax*s_temp), y0-1);
      
      g.drawLine(x0+(int)(stepMin*s_temp), y0-5, x0+(int)(stepMin*s_temp), y0+5);
      g.drawString(Integer.toString((int)stepMin), x0+(int)(stepMin*s_temp), y0-1);

    }
            
            
      public void plotArc(Graphics2D g, double dec, double inc, double d, double x0, double y0){//d->diameter x0,y0->center
                          
         CircunferenceArc arc = new CircunferenceArc(dec, inc, d/2, x0, y0);
         Arc2D gArc = new Arc2D.Double(
                arc.getX0()-arc.getR0(), arc.getY0()-arc.getR0(), //left upper corner
                arc.getR0()*2, arc.getR0()*2, 
                arc.getStart(), arc.getExtent(), Arc2D.OPEN);
         g.draw(gArc);
         arc = new CircunferenceArc(dec+180, inc, d/2, x0, y0);
         gArc = new Arc2D.Double(
                arc.getX0()-arc.getR0(), arc.getY0()-arc.getR0(), //left upper corner
                arc.getR0()*2, arc.getR0()*2, 
                arc.getStart(), arc.getExtent(), Arc2D.OPEN);
         g.draw(gArc);
       
     }
}


